// FlexUtil.h : VSFlexGrid Pro 6.0 utilities
//

#include "flexgrid.h"


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Unit conversion utilities
//
long XTwipsToPixels(long twips);
long YTwipsToPixels(long twips);
long XPixelsToTwips(long pixels);
long YPixelsToTwips(long pixels);


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// HTML generation
//
BOOL SaveGridToHTML(CvsFlexGrid* ctl, LPSTR fname);


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// CvsFlexGridDataObject wrapper class
//
// this is useful for handling the data object passed as a parameter in
// OLE drag drop events.
//
// for example:
//
// void CBigDemoDlg::OnOLEDragDrop(LPDISPATCH FAR* Data, ...)
// {
//		// wrap data object's IDispatch
//		CvsFlexGridDataObject ddData(*Data);
//
//		// make sure the object has files
//		BOOL bHasFiles = (ddData.GetFileCount() > 0);
//	...
// }
//

#ifndef __CVSFLEXGRIDDATAOBJECT__
#define __CVSFLEXGRIDDATAOBJECT__

class CvsFlexGridDataObject : public COleDispatchDriver
{
public:

	CvsFlexGridDataObject(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch, FALSE) {}

	void Clear()
	{
		InvokeHelper(1, DISPATCH_METHOD, VT_EMPTY, NULL, NULL);
	}
	short GetFileCount()
	{
		BOOL result;
		InvokeHelper(2, DISPATCH_PROPERTYGET, VT_BOOL, (void*)&result, NULL);
		return result;
	}
	VARIANT GetData(short Format)
	{
		VARIANT result;
		static BYTE parms[] = VTS_VARIANT;
		InvokeHelper(3, DISPATCH_PROPERTYGET, VT_BOOL, (void*)&result, parms, Format);
		return result;
	}
	VARIANT_BOOL GetFormat(short Format)
	{
		BOOL result;
		static BYTE parms[] = VTS_I2;
		InvokeHelper(4, DISPATCH_PROPERTYGET, VT_BOOL, (void*)&result, parms, Format);
		return result;
	}
	CString GetFiles(short Index)
	{
		CString result;
		static BYTE parms[] = VTS_I2;
		InvokeHelper(5, DISPATCH_PROPERTYGET, VT_BSTR, (void*)&result, parms, Index);
		return result;
	}
	void SetData(const VARIANT& Data, short Format = 0)
	{
		static BYTE parms[] = VTS_VARIANT VTS_I2;
		InvokeHelper(6, DISPATCH_PROPERTYPUT, VT_EMPTY, NULL, parms, &Data, Format);
	}
};
#endif // __CVSFLEXGRIDDATAOBJECT__


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Enumeration constants for VSFlexGrid Pro 6.0
//
typedef enum AppearanceSettings {
    flexFlat,
    flex3D,
} AppearanceSettings;

typedef enum BorderStyleSettings {
    flexBorderNone,
    flexBorderFlat,
} BorderStyleSettings;

typedef enum MousePointerSettings {
    flexDefault,
    flexArrow,
    flexCross,
    flexIBeam,
    flexIcon,
    flexSize,
    flexSizeNESW,
    flexSizeNS,
    flexSizeNWSE,
    flexSizeEW,
    flexUpArrow,
    flexHourglass,
    flexNoDrop,
    flexArrowHourGlass,
    flexArrowQuestion,
    flexSizeAll,
    flexCustom = 99,
} MousePointerSettings;

typedef enum RowStatusSettings {
	flexrsNew,
	flexrsUpdated,
	flexrsModified,
	flexrsDeleted,
} RowStatusSettings;

typedef enum CellPropertySettings {
	flexcpText,
	flexcpTextStyle,
	flexcpAlignment,
	flexcpPicture,
	flexcpPictureAlignment,
	flexcpChecked,
	flexcpBackColor,
	flexcpForeColor,
	flexcpFloodPercent,
	flexcpFloodColor,
	flexcpFont,
	flexcpFontName,
	flexcpFontSize,
	flexcpFontBold,
	flexcpFontItalic,
	flexcpFontUnderline,
	flexcpFontStrikethru,
	flexcpFontWidth,
	flexcpValue,
	flexcpTextDisplay,
	flexcpData,
	flexcpCustomFormat,
} CellPropertySettings;

typedef enum ExplorerBarSettings {
	flexExNone,
	flexExSort,
	flexExMove,
	flexExSortAndMove,
} ExplorerBarSettings;

typedef enum AutoSearchSettings {
	flexSearchNone,
	flexSearchFromTop,
	flexSearchFromCursor,
} AutoSearchSettings;

typedef enum MergeCompareSettings {
	flexMCExact,
	flexMCNoCase,
	flexMCTrimNoCase,
} MergeCompareSettings;

typedef enum EllipsisSettings {
	flexNoEllipsis,
	flexEllipsisEnd,
	flexEllipsisPath,
} EllipsisSettings;

typedef enum CellCheckedSettings {
	flexNoCheckbox,
	flexChecked,
	flexUnchecked,
} CellCheckedSettings;

typedef enum TabBehaviorSettings {
    flexTabControls,
    flexTabCells,
} TabBehaviorSettings;

typedef enum SaveLoadSettings {
    flexFileAll,
    flexFileData,
    flexFileFormat,
    flexFileCommaText,
    flexFileTabText,
} SaveLoadSettings;

typedef enum ArchiveSettings {
	arcAdd,
	arcStore,
	arcExtract,
	arcDelete,
} ArchiveSettings;

typedef enum ArchiveInfoSettings {
	arcFileCount,
	arcFileName,
	arcFileSize,
	arcFileCompressedSize,
	arcFileDate
} ArchiveInfoSettings;

typedef enum OLEDragModeSettings {
    flexOLEDragManual,
    flexOLEDragAutomatic,
} OLEDragModeSettings;

typedef enum OLEDropModeSettings {
    flexOLEDropNone,
    flexOLEDropManual,
    flexOLEDropAutomatic,
} OLEDropModeSettings;

typedef enum DataModeSettings {
    flexDMFree,
    flexDMBound,
} DataModeSettings;

typedef enum OwnerDrawSettings {
    flexODNone,
    flexODOver,
    flexODContent,
    flexODComplete,
    flexODOverFixed,
    flexODContentFixed,
    flexODCompleteFixed,
} OwnerDrawSettings;

typedef enum DataTypeSettings {
	flexDTEmpty = 0,
	flexDTShort = 2,
	flexDTLong = 3,
	flexDTSingle = 4,
	flexDTDouble = 5,
	flexDTCurrency = 6,
	flexDTDate = 7,
	flexDTString = 8,
	flexDTBoolean = 11,
	flexDTLong8 = 20,
	flexDTStringC = 30,
	flexDTStringW = 31,
} DataTypeSettings;

typedef enum AutoSizeSettings {
    flexAutoSizeColWidth,
    flexAutoSizeRowHeight,
} AutoSizeSettings;

typedef enum CollapsedSettings {
    flexOutlineExpanded,
    flexOutlineSubtotals,
    flexOutlineCollapsed,
} CollapsedSettings;

typedef enum OutlineBarSettings {
    flexOutlineBarNone,
    flexOutlineBarComplete,
    flexOutlineBarSimple,
    flexOutlineBarSymbols,
} OutlineBarSettings;

typedef enum SubtotalSettings {
    flexSTNone,
    flexSTClear,
    flexSTSum,
    flexSTPercent,
    flexSTCount,
    flexSTAverage,
    flexSTMax,
    flexSTMin,
    flexSTStd,
    flexSTVar,
} SubtotalSettings;

typedef enum SortSettings {
    flexSortNone,
    flexSortGenericAscending,
    flexSortGenericDescending,
    flexSortNumericAscending,
    flexSortNumericDescending,
    flexSortStringNoCaseAscending,
    flexSortStringNoCaseDescending,
    flexSortStringAscending,
    flexSortStringDescending,
    flexSortCustom,
    flexSortUseColSort,
} SortSettings;

typedef enum FocusRectSettings {
    flexFocusNone,
    flexFocusLight,
    flexFocusHeavy,
    flexFocusSolid,
    flexFocusRaised,
    flexFocusInset,
} FocusRectSettings;

typedef enum ShowSelSettings {
    flexHighlightNever,
    flexHighlightAlways,
    flexHighlightWithFocus,
} ShowSelSettings;

typedef enum ScrollBarsSettings {
    flexScrollBarNone,
    flexScrollBarHorizontal,
    flexScrollBarVertical,
    flexScrollBarBoth,
} ScrollBarsSettings;

typedef enum TextStyleSettings { 
    flexTextFlat,
    flexTextRaised,
    flexTextInset,
    flexTextRaisedLight,
    flexTextInsetLight,
} TextStyleSettings;

typedef enum FillStyleSettings {
    flexFillSingle,
    flexFillRepeat,
} FillStyleSettings;

typedef enum GridStyleSettings {
    flexGridNone,
    flexGridFlat,
    flexGridInset,
    flexGridRaised,
    flexGridFlatHorz,
    flexGridInsetHorz,
    flexGridRaisedHorz,
    flexGridSkipHorz,
    flexGridFlatVert,
    flexGridInsetVert,
    flexGridRaisedVert,
    flexGridSkipVert,
    flexGridExplorer,
} GridStyleSettings;

typedef enum SelModeSettings {
    flexSelectionFree,
    flexSelectionByRow,
    flexSelectionByColumn,
    flexSelectionListBox,
} SelModeSettings;

typedef enum MergeSettings {
    flexMergeNever,
    flexMergeFree,
    flexMergeRestrictRows,
    flexMergeRestrictColumns,
    flexMergeRestrictAll,
    flexMergeFixedOnly,
    flexMergeSpill,
} MergeSettings;

typedef enum PictureTypeSettings {
    flexPictureColor,
    flexPictureMonochrome,
} PictureTypeSettings;

typedef enum AllowUserResizeSettings {
    flexResizeNone,
    flexResizeColumns,
    flexResizeRows,
    flexResizeBoth,
} AllowUserResizeSettings;

typedef enum SubtotalPositionSettings {
    flexSTBelow,
    flexSTAbove,
} SubtotalPositionSettings;

typedef enum AlignmentSettings {
    flexAlignLeftTop,
    flexAlignLeftCenter,
    flexAlignLeftBottom,
    flexAlignCenterTop,
    flexAlignCenterCenter,
    flexAlignCenterBottom,
    flexAlignRightTop,
    flexAlignRightCenter,
    flexAlignRightBottom,
    flexAlignGeneral,
} AlignmentSettings;

typedef enum PictureAlignmentSettings {
    flexPicAlignLeftTop,
    flexPicAlignLeftCenter,
    flexPicAlignLeftBottom,
    flexPicAlignCenterTop,
    flexPicAlignCenterCenter,
    flexPicAlignCenterBottom,
    flexPicAlignRightTop,
    flexPicAlignRightCenter,
    flexPicAlignRightBottom,
    flexPicAlignStretch,
    flexPicAlignTile,
} PictureAlignmentSettings;

typedef enum StringErrorSettings {
    flexErrNone,
    flexErrOutOfMemory,
    flexErrSquareB,
    flexErrCurlyB,
    flexErrBadPattern,
    flexErrBadTagIndex,
    flexErrNoMatch,
    flexErrInvalidMatchIndex,
} StringErrorSettings;
